;; The home module is the module which will contain the definitions for
;; the external type objects.  If this file is included in some other
;; module only the external declarations will be generated.  Unless the
;; set-home-module! macro is used the definitions are generated.

;; Here we define a link to the memcpy function which is used to 
;; do structure assignments.

;(define-c-builtin (memcpy* pointer pointer unsigned) pointer "memcpy")
;(define-in-line
;  (memcpy dest src n)
;  (let* ((dest (cdr dest))
;	 (src (cdr src))
;	 (n n)
;	 (return-value (cons 'void* (memcpy* dest src n))))
;    return-value))

; First we create a compile time variable with the current module name.

(eval-when
 (compile eval)

  ;;; Converts a list of strings or symbols into an upper-case
  ;;; uninterned symbol.

  (define (UIS . syms)
    (string->uninterned-symbol
     (list->string
      (let loop ((syms syms))
	(if (null? syms)
	    '()
	    (append (map char-upcase
			 (string->list
			  (if (symbol? (car syms))
			      (symbol->string (car syms))
			      (car syms))))
		    (loop (cdr syms))))))))

  ;;; Converts a list of strings or symbols into an upper-case
  ;;; interned symbol.

  (define (IS . syms)
    (string->symbol
     (list->string
      (let loop ((syms syms))
	(if (null? syms)
	    '()
	    (append (map char-upcase
			 (string->list
			  (if (symbol? (car syms))
			      (symbol->string (car syms))
			      (car syms))))
		    (loop (cdr syms)))))))))

;(eval-when
; (compile)
;  (define *module* (is module-name))
;  (define *home-module-stack* (list (is module-name))))
;
;(define-macro (push-home-module! name)
;  (set! *home-module-stack* (cons name *home-module-stack*))
;  #t)
;
;(define-macro (pop-home-module!)
;  (set! *home-module-stack* (cdr *home-module-stack*))
;  #t)

; If this variable is set to true during compilation definitions will
; be generated to make functions and variables which will be available
; from the REPL loop for each extern function and const declaration.

(eval-when
 (compile eval)
 (define *top-level-externs* #f))

(define-macro (set-top-level-externs! value)
  (set! *top-level-externs* value)
  #t)

(eval-when
 (compile eval)
 (begin

   ;;; MACHINE DEPENDENT SIZEOF declarations are processed here.

   (define (INPUT-SIZEOF type size align to-get to-set!)
     (putprop type 'type #t)
     (putprop type 'base-type type)
     (putprop type 'size size)
     (putprop type 'align align)
     (putprop type 'to-get to-get)
     (putprop type 'to-set! to-set!))

   ;;; These need to be integrated back into the compiler.

   (input-sizeof 'char (sizeof-char) (alignof-char) 'c-byte-ref 'c-byte-set!)
   ;(input-sizeof 'bool (sizeof-bool) (alignof-bool) 'c-bool-ref 'c-bool-set!)
   (input-sizeof 'shortint (sizeof-shortint) (alignof-shortint)
		 'c-shortint-ref 'c-shortint-set!)
   (input-sizeof 'shortunsigned (sizeof-shortunsigned) (alignof-shortunsigned)
		 'c-shortunsigned-ref 'c-shortunsigned-set!)
   (input-sizeof 'int (sizeof-int) (alignof-int)
		 'c-int-ref 'c-int-set!)
   (input-sizeof 'unsigned (sizeof-unsigned) (alignof-unsigned)
		 'c-unsigned-ref 'c-unsigned-set!)
   (input-sizeof 'longint (sizeof-longint) (alignof-longint)
		 'c-longint-ref 'c-longint-set!)
   (input-sizeof 'longunsigned (sizeof-longunsigned) (alignof-longunsigned)
		 'c-longunsigned-ref 'c-longunsigned-set!)
   (input-sizeof 'float (sizeof-float) (alignof-float)
		 'c-float-ref 'c-float-set!)
   (input-sizeof 'double (sizeof-double) (alignof-double)
		 'c-double-ref 'c-double-set!)
   (input-sizeof 'pointer (sizeof-pointer) (alignof-pointer)
		 (pointer-ref) (pointer-set!))
   (input-sizeof 'procedure (sizeof-procedure) (alignof-procedure)
		 (procedure-ref) (procedure-set!))

  (define (PARSE-TYPE type)
    (if (pair? type)
	(cond ((memq (car type) '(struct union))
	       (struct-or-union type))
	      ((equal? (cdr type) '(*))
	       (list (parse-stype (car type)) '*))
	      ((equal? (cdr type) '(*proc))
	       (list (parse-stype (car type)) '*proc))
	      (else (parse-atype type)))
	(parse-stype type)))

  (define (PARSE-ATYPE type)
    (if (pair? type)
	(let ((dim #f))
	  (cond ((memq (car type) '(struct union))
		 (struct-or-union type))
		((begin
		   (set! dim (and (= (length type) 2) (expand (cadr type))))
		   (and (integer? dim) (>= dim 0)))
		 (list (parse-stype (car type)) dim))
		(else (error 'parse-atype "Argument is not a legal type: ~s"
			     type))))
	(parse-stype type)))

  (define (PARSE-STYPE type)
    (if (symbol? type)
	type
	(error 'parse-stype "Argument is not a legal type: ~s" type)))

  ;;; Structs and unions are handled by the following functions.

  (define (STRUCT-OR-UNION exp)
    (list (case (car exp)
	    ((struct) 'struct)
	    ((union) 'union)
	    (else (error 'struct-or-union "Illegal syntax: ~s" exp)))
	  (map (lambda (slot)
		 (if (and (= (length slot) 2) (symbol? (cadr slot)))
		     (list (parse-slot-type (car slot)) (cadr slot))
		     (error 'struct-or-union
			    "Argument is not a legal slot: ~s" slot)))
	       (cdr exp))))

  ;;; When the type specifier for a slot is parsed, it may be contain an
  ;;; array or structure definition, or a symbol.  Arrays and structures
  ;;; defined here must have a dummy type assigned to them.

  (define PARSE-SLOT-TYPE
    (let ((uid 1))
      (lambda (type)
	(let ((parse (parse-atype type)))
	  (if (symbol? parse)
	      parse
	      (let ((symbol
		     (string->symbol (format "*TYPE~s" uid))))
		(set! uid (+ uid 1))
		(putprop symbol 'base-type #f)
		(putprop symbol 'type parse)
		symbol))))))

  ;;; Every type symbol can be resolved into a base type symbol by
  ;;; the following function.  Once a base type has been computed,
  ;;; it is saved on the property list.

  (define (BASE-TYPE start-type)
    (or (getprop start-type 'base-type)
	(let loop ((type start-type) (count 20))
	  (let ((typeinfo (getprop type 'type)))
	    (if (or (not typeinfo) (eq? count 0))
		(error 'base-type "BASE TYPE cannot be resolved: ~s"
		       start-type))
	    (if (symbol? typeinfo)
		(loop typeinfo (- count 1))
		(putprop start-type 'base-type type))))))

  ;;; Basic information about a type is returned by:

  (define (ISA-UNION? type)
    (let ((typeinfo (getprop (base-type type) 'type)))
      (and (pair? typeinfo) (eq? (car typeinfo) 'union))))

  (define (ISA-STRUCT? type)
    (let ((typeinfo (getprop (base-type type) 'type)))
      (and (pair? typeinfo) (eq? (car typeinfo) 'struct))))

  (define (UORS-SLOTS type) (cadr (getprop (base-type type) 'type)))

  (define (ISA-PROCP? type)
    (let ((typeinfo (getprop (base-type type) 'type)))
      (and (pair? typeinfo) (eq? (cadr typeinfo) '*proc))))

  (define (PROCP-RETURNS type)
    (base-type (car (getprop (base-type type) 'type))))

  (define (ISA-POINTER? type)
    (let ((typeinfo (getprop (base-type type) 'type)))
      (and (pair? typeinfo) (eq? (cadr typeinfo) '*))))

  (define (POINTER-TO type)
    (base-type (car (getprop (base-type type) 'type))))

  (define (ISA-ARRAY? type)
    (let ((typeinfo (getprop (base-type type) 'type)))
      (and (pair? typeinfo) (number? (cadr typeinfo)))))

  (define (ARRAY-SIZE type) (cadr (getprop (base-type type) 'type)))

  (define (ARRAY-TYPE type)
    (base-type (car (getprop (base-type type) 'type))))

  (define (POINTED-TO-BY type)
    (let* ((base (base-type type))
	   (prop (and base (getprop base 'pointed-to-by))))
      (and prop (base-type prop))))

  ;;; Given this information, we can now compute sizes of things.

  (define (ALIGNED-SIZE-OF type)
    (let* ((size-align (size-alignment-of type))
	   (size (car size-align))
	   (align (cadr size-align)))
      (* align (quotient (+ size align -1) align))))

  (define (SIZE-OF type)
    (car (size-alignment-of type)))

  (define (ALIGNMENT-OF type)
    (cadr (size-alignment-of type)))

  (define (SIZE-ALIGNMENT-OF type)
    (let ((base (base-type type)))
      (cond ((getprop base 'size)
	     `(,(getprop base 'size) ,(getprop base 'align)))
	    ((isa-union? type)
	     (let ((size 0)
		   (align 0))
	       (for-each
		(lambda (slot)
		  (let ((size-align (size-alignment-of
				     (car slot))))
		    (set! size (max size
				    (car size-align)))
		    (set! align (max align
				     (cadr size-align)))))
		(uors-slots type))
	       `(,size ,align)))
	    ((isa-struct? type)
	     (let ((size 0)
		   (align 0))
	       (for-each
		(lambda (slot)
		  (let* ((size-align (size-alignment-of
				      (car slot)))
			 (ssize (car size-align))
			 (salign (cadr size-align)))
		    (if (not (zero? (remainder size
					       salign)))
			(set! size
			      (+ size
				 (- salign (remainder size
						      salign)))))
		    (set! size (+ size ssize))
		    (set! align (max align salign))))
		(uors-slots type))
	       `(,size ,align)))
	    ((isa-procp? type) (size-alignment-of 'procedure))
	    ((isa-pointer? type) (size-alignment-of 'pointer))
	    ((isa-array? type)
	     (let ((size-align (size-alignment-of (array-type type))))
	       `(,(* (array-size type) (car size-align))
		 ,(cadr size-align))))
	    (else (error 'size-alignment-of "Mystery type: ~s" type)))))

  ;;; A method for loading a type which takes an object, an offset, and an
  ;;; index (only for arrays) as it's arguments is returned by the following
  ;;; function.

  (define (TO-GET-TYPE type)
    (let ((base (base-type type)))
      (cond ((getprop base 'to-get))
	    ((isa-array? base)
	     `(lambda (x y i)
		(,(to-get-type (array-type base)) x
		 (+ y (* ,(size-of (array-type base)) i)))))
	    ((isa-pointer? base)
	     `(lambda (x y)
		(cons ',base
		      (,(to-get-type 'pointer) x y))))
	    ((isa-procp? base)
	     `(lambda (x y)
		(cons ',base (,(to-get-type 'pointer) x y))))
	    (else #f))))

  ;;; A method for storing a type which takes an object, an offset, an index
  ;;; (only for arrays), and a new value as it's arguments is returned by the
  ;;; following function.

  (define (TO-SET!-TYPE type)
    (let ((base (base-type type)))
      (cond ((getprop base 'to-set!))
	    ((isa-array? base)
	     `(lambda (x y i z)
		(,(to-set!-type (array-type base)) x
		 (+ y (* ,(size-of (array-type base)) i))
		 z)))
	    ((isa-pointer? base)
	     `(lambda (x y z)
		(,(to-set!-type 'pointer) x y
		 (,(to-check-type  base) z))))
	    ((isa-procp? base)
	     `(lambda (x y z)
		(,(to-set!-type 'pointer) x y
		 (,(to-check-type base) z))))
	    (else #f))))

  ;;; A method for checking a type and returning the "raw" value
  ;;; which takes an object as it's argument is returned by the
  ;;; following function.

  (define (TO-CHECK-TYPE type)
    (let ((base (base-type type)))
      (if (or (isa-pointer? base) (isa-procp? base))
	  (is "CHK-" base)
	  '(lambda (x) x))))

  ;;; The symbol that is used as the type tag for objects is returned by the
  ;;; following procedure.  It returns #f when there is no type tag.

  (define (TYPE-TAG type)
    (let ((base (base-type type)))
      (if (or (isa-pointer? base) (isa-procp? base))
	  base
	  #f)))

    ;;; Checking functions for procedure pointer types are emitted by
    ;;; the following procedure.  The arguments are the object type
    ;;; and the procedure to print the definitions.

  (define (EMIT-CHK-PROCS type)
    (list
     `(define-in-line (,(is "CHK-" (type-tag type)) x)
	(if (and (pair? x) (eq? (car x) ',(type-tag type)))
	    (cdr x)
	    (error ',(is "CHK-" (type-tag type))
		   "Incorrect argument type: ~s" x)))
     `(define-in-line (,(is "ISA-" (type-tag type) "?") x)
	(and (pair? x) (eq? (car x) ',(type-tag type))))

;       `(define-macro (,(uis "ISA-" (type-tag type) "?") x)
;	  (list
;	   'quasiquote
;	   (list
;	    'and
;	    (list 'pair 'x)
;	    (list 'eq? (list 'car 'x) ',(type-tag type)))))

     ))

    ;;; Access functions for array types are generated by the
    ;;; following procedure.  The arguments are the object type, a
    ;;; read-only flag, and the function to print the definitions.

  (define (EMIT-ARRAY-PROCS pointer read-only defform)
    (let* ((type (pointer-to pointer))
	   (size (array-size type))
	   (entry-type (array-type type))
	   (chk (to-check-type pointer)))

      (cond ((getprop type 'array-procs)
	     '())
	    (else
	     (putprop type 'array-procs #t)

	     (cons
	      `(,defform (,(is type "-LENGTH") x)
		 (quotient (string-length (,chk x))
			   ,(size-of entry-type)))
	      (cond ((or (isa-struct? entry-type) (isa-union? entry-type))
		     (list
		      `(,defform (,(is type "->" entry-type "-LIST") x)
			 (let* ((array (,chk x))
				(asize (string-length array))
				(esize ,(size-of entry-type)))
			   (let loop ((x 0))
			     (if (eq? x asize)
				 '()
				 (cons (cons ',(pointed-to-by
						entry-type)
					     (substring array
							x (+ x esize)))
				       (loop (+ x esize)))))))
		      `(,defform (,(is entry-type "-LIST->" type) x)
			 (cons ',pointer
			       (apply string-append
				      (map (lambda (x) (,(to-check-type
							  (pointed-to-by
							   entry-type)) x))
					   x))))))
		    (else
		     (list
		      `(,defform (,type x i)
			 (,(to-get-type type) (,chk x) 0 i))
		      `(,defform (,(is type "->" entry-type "-LIST") x)
			 (let loop ((i 0)
				    (count (,(is type "-LENGTH") x)))
			   (if (eq? i count)
			       '()
			       (cons (,type x i)
				     (loop (+ i 1) count)))))
		      `(,defform (,(is entry-type "-LIST->" type) l)
			 (let loop ((l l)
				    (i 0)
				    (a (,(is "MAKE-" type)
					,@(if (eq? size 0)
					      '((length l))
					      '()))))
			   (if (null? l)
			       a
			       (begin (,(is type "!") a i (car l))
				      (loop (cdr l) (+ i 1) a)))))
		      `(,defform (,(is type "!") x i z)
			 (,(to-set!-type type) (,chk x) 0 i z))
		      `(,defform (,(is "MAKE-" type)
				  ,@(if (eq? size 0) '(x) '()))
			 (cons ',pointer
			       (make-string
				(* ,(size-of entry-type)
				   ,(if (eq? size 0) 'x	size))
				(integer->char 0))))))))

	     ))

      ))

    ;;; Write the source file containing the struct definition.  

  (define (EMIT-STRUCT-PROCS pointer read-only type defform)
    (let* ((slots (uors-slots type)))
      (if (null? slots)
	  '()
	  (cons
	   `(,defform (,(is "MAKE-" type))
	      (cons ',pointer
		    (make-string
		     ,(aligned-size-of type)
		     (integer->char 0))))
	   (slot-getset type type 0 pointer read-only defform)))))

    ;;; Slot access functions for a structure are created by the following
    ;;; function.

  (define (SLOT-GETSET type preamble offset base read-only defform)

    ;; To obtain a pointer to a structure member we return a pair,
    ;; with the string holding the structure in the car and the offset
    ;; in the cdr.  Note that if the type of the returned pointer has
    ;; not been previously declared this will emit a procedure that
    ;; calls error.
    
    (define (EMIT-STRUCT-POINTER type slot-type name offset)
      (let ((pointer (pointed-to-by slot-type)))
	(cond
	 ;; I haven't thought about dealing with arrays yet.
	 ((or (not type) (isa-array? slot-type))
	  '())
	 ;; The type to be returned has not been declared - emit an error
	 ((not pointer)
	  `((,defform (,(is "&" preamble "-" name) x)
	      (error ',(is "&" preamble "-" name)
		     "Can't return undeclared type (~s *)~%"
		     ',(base-type slot-type)))))
	 (else
	  `((,defform (,(is "&" preamble "-" name) x)
	      (let ((raw (,(to-check-type type) x)))
		(cons ',pointer
		      (if (pair? raw)
			  (cons (car raw) (+ (cdr raw) ,offset))
			  (cons raw ,offset))))))
	  ))))

    ;; Copy the value of a structure.

    (define (EMIT-STRUCT-SETTER type slot-type name offset)
      (if (pointed-to-by slot-type)
	  `((,defform (,(is preamble "-" name "!") x y)
	      (memcpy* (cons (,(to-check-type type) x) ,offset)
		       (,(to-check-type (pointed-to-by slot-type)) y)
		       ,(size-of slot-type))))
	  '()))

    ;; Emit the access procedures for a slot whose type is not a
    ;; struct or a union.
    
    (define (EMIT-PROCS type name offset)
      (let* ((index (if (isa-array? type) '(i) '())))
	(append
	 `((,defform (,(is preamble "-" name) x ,@index)
	     (,(to-get-type type)
	      (,(to-check-type base) x)
	      ,offset
	      ,@index)))
	 (if (not read-only)
	     `((,defform (,(is preamble "-" name "!") x ,@index y)
		 (,(to-set!-type type)
		  (,(to-check-type base) x)
		  ,offset
		  ,@index
		  y)))
	     '())

	 (emit-struct-pointer base type name offset))))

    (let loop ((procs '()) (slots (uors-slots type)) (offset offset))
      (if (null? slots)
	  procs
	  (let* ((slot-type (caar slots))
		 (slot-name (cadar slots))
		 (size-align (size-alignment-of slot-type))
		 (size (car size-align))
		 (align (cadr size-align))
		 (newprocs '()))
	    ;; Align the beginning of the slot
	    (set! offset (let ((tmp (+ offset align -1)))
			   (- tmp (modulo tmp align))))
	    (loop
	     (append
	      procs
	      (cond ((or (isa-union? slot-type) (isa-struct? slot-type))
		     (append
		      (emit-struct-pointer base slot-type slot-name offset)
		      (emit-struct-setter base slot-type slot-name offset)
		      (slot-getset slot-type
				   (is preamble "-" slot-name)
				   offset base read-only defform)))
		    (else
		     (emit-procs slot-type slot-name offset))))
	     (cdr slots)
	     (if (isa-union? type)
		 offset
		 (let ((tmp (+ size offset align -1))) (- tmp (modulo tmp align)))))))))

  ;; When a pointer type is declared this procedure emits procedures
  ;; to create instances of the object it refers to, and to set and
  ;; dereference the pointer.

  (define (EMIT-POINTER-PROCS pointer read-only defform)
    (let ((type (car (getprop (base-type pointer) 'type))))
      (if (getprop type 'procs)
	  '()
	  (begin
	    (putprop type 'procs #t)
	    (append
	     `((,defform (,(is "MAKE-" type))
		 (cons ',pointer
		       (make-string ,(aligned-size-of type)
				    (integer->char 0))))
	       (,defform (,(is type "-REF") x)
		 (,(to-get-type type) (cdr x) 0)))
	     (if read-only
		 '()
		 `((,defform (,(is type "-SET!") x y)
		     (,(to-set!-type type) (cdr x) 0 y)))))))))

    ;;; Emit the code for a type.

  (define (EMIT-TYPE type read-only defform)
    (cond
     ((isa-pointer? type)
      `(begin
	 ,@(append
	    (emit-chk-procs type)
	    (cond
	     ((or (isa-union? (pointer-to type))
		  (isa-struct? (pointer-to type)))
	      (let* ((struct (pointer-to type)))
		(emit-struct-procs type read-only struct defform)))
	     ((isa-array? (pointer-to type))
	      (let ((result 
		     (emit-array-procs type read-only defform)))
		;;(format #t "(emit-array-procs ~s ~s ~s)~% -> ~s~%~%"
		;;type read-only defform result)
		result))
	     (else
	      (emit-pointer-procs type read-only defform))))))
	  ((isa-procp? type)
	   `(begin ,@(emit-chk-procs type)))
	  (else #t)))

  (define (PARSE-ARG exp)
    (if (and (pair? exp)
	     (or (and (= (length exp) 2)
		      (parse-type (car exp))
		      (symbol? (cadr exp)))
		 (and (= (length exp) 3)
		      (memq (car exp) '(in out in_out))
		      (parse-type (cadr exp))
		      (symbol? (caddr exp)))))
	#t
	(error 'PARSE-ARG "Illegal ARGUMENT syntax: ~s" exp)))

  ;;; Converts the type of a procedure argument to a simple C-type.

  (define (SIMPLE-TYPE type)
    (cond ((memq (car type) '(in out in_out string)) 'pointer)
	  ((eq? (car type) 'void) 'void)
	  ((isa-pointer? (car type)) 'pointer)
	  ((isa-procp? (car type)) 'pointer)
	  (else (base-type (car type)))))

  ;;; Called to do input conversion for arguments.  Return an expression
  ;;; of th form (<var> <value>).

  (define (ARG-IN arg)
    (let* ((flag (if (memq (car arg) '(in out in_out))
		     (car arg)
		     #f))
	   (type (if flag (cadr arg) (car arg)))
	   (var  (if flag (caddr arg) (cadr arg))))
      (case flag
	((in in_out) (cond ((eq? (base-type type) 'int)
			    `(,var (let ((_buf (make-string
						,(size-of 'int))))
				     (c-int-set! _buf 0 ,var)
				     _buf)))
			   (else `(,var (in->c ,var)))))
	((out) `(,var (make-string ,(if (eq? type 'string)
					(size-of 'pointer)
					(aligned-size-of type)))))
	(else (cond ((eq? type 'string)
		     `(,var (if (string? ,var)
				,var
				(error 'chk-string
				       "Argument is incorrect type: ~s"
				       ,var))))
		    ((isa-pointer? type)
		     `(,var (,(is "CHK-" (base-type type)) ,var)))
		    (else  `(,var ,var)))))))

  ;;; Return a list of the expressions required to do output conversion after
  ;;; an external call.
	 
  (define (ARGS-OUT args)
    
    (define (ARG-OUT arg)
      (let* ((flag (if (memq (car arg) '(in out in_out))
		       (car arg)
		       #f))
	     (type (if flag (cadr arg) (car arg)))
	     (var  (if flag (caddr arg) (cadr arg))))
	(case flag
	  ((in) #f)
	  ((in_out out)
	   (cond ((eq? type 'bool)
		  `(not (= (c-unsigned-ref ,var 0) 0)))
		 ((eq? type 'string)
		  `(c-string->string (c-unsigned-ref ,var 0)))
		 ((isa-pointer? type)
		  `(cons ',(base-type type)
			 (c-unsigned-ref ,var 0)))
		 ((or (isa-union? type) (isa-struct? type)
		      (isa-array? type))
		  `(cons ',(pointed-to-by type) ,var))
		 (else `(,(getprop (base-type type) 'to-get)
			 ,var 0))))
	  (else #f))))

    (if (null? args)
	'()
	(let ((out (arg-out (car args))))
	  (if out
	      (cons out (args-out (cdr args)))
	      (args-out (cdr args))))))))

(define-macro (const id value)
  `(begin
     ,(if *top-level-externs* `(define ,id ,value) #t)
     (define-constant ,id ,value)))

;;; Type declarations are parsed by the following macro.  It expands
;;; to a set of macros which define the the type, or call error on an
;;; error.  Some of these type transformations may be MACHINE DEPENDENT.
;;; It also adds entries to the type name's property list, including
;;;	base-type - 
;;;	type - 
;;;	

(define-macro (typedef type id)
  (let ((parse (parse-type type))
	(oldparse (getprop id 'type)))
    (cond
     ((not oldparse)
      (putprop id 'base-type #f)
      (putprop id 'type parse)
      (if (and (pair? parse) (symbol? (car parse)) (eq? (cadr parse) '*))
	 (putprop (car parse) 'pointed-to-by id))
      (emit-type id #f (if *top-level-externs* 'define 'define-in-line)))
     ((equal? parse oldparse)
      #t)
     (else
      (format stderr-port
	      "TYPEDEF WARNING: Conflicting types for ~s: ~s vs. ~s~%" 
	     id oldparse parse)
      #t))))

;(eval-when
; (compile)
;  (define (EMIT-EXTERN-foo rettype cname args defform)
;    (let ((xname (is cname "*")))
;      
;      (define (EMIT-CALL)
;	`(,xname ,@(map (lambda (x) (car (last-pair x))) args)))
;      
;      (define (FORMALS args)
;	(if (null? args)
;	    '()
;	    (if (eq? (caar args) 'out)
;		(formals (cdr args))
;		(cons (car (last-pair (car args)))
;		      (formals (cdr args))))))
;      
;;      (if (eq? *module* (car *home-module-stack*))
;      `(begin
;	 (define-c-external
;	   (,xname ,@(map simple-type args))
;	   ,(simple-type (list rettype 'returned))
;	   ,cname)
;
;	 (,defform (,(is cname) ,@(formals args))
;	   (let* (,@(map arg-in args)
;		  (return-value
;		   ,(cond ((eq? rettype 'void)
;			   `(begin ,(emit-call) #f))
;			  ((eq? rettype 'string)
;			   `(c-string->string ,(emit-call)))
;			  ((isa-pointer? rettype)
;			   `(cons ',(base-type rettype)
;				  ,(emit-call)))
;			  (else (emit-call)))))
;	     ,(let ((out (args-out args)))
;		(if (null? out)
;		    'return-value
;		    (if (eq? rettype 'void)
;			(if (= (length out) 1)
;			    (car out)
;			    `(list ,@out))
;			`(list return-value ,@out)))))))
;
;	  ; else
;;	  (let ((formals (let loop ((args args)
;;				    (formals '(a b c d e f g h i j k l m
;;						 n o p q r s t u v w x y z)))
;;			   (cond ((null? args) '())
;;				 ((eq? (caar args) 'out)
;;				  (loop (cdr args) (cdr formals)))
;;				 (else
;;				  (cons (car formals)
;;					(loop (cdr args) (cdr formals))))))))
;;	    
;;	    `(define-external (,(is cname) ,@formals)
;;	       ,(car *home-module-stack*)))
;;	  )
;      )))

(define-macro (do-extern builtin? rtype name . args)
  (let ((result
	 (cond ((and (parse-type rtype)

		     ; We need the name of the C symbol, the name of the
		     ; Scheme function we are producing, and a name for the
		     ; intermediate scheme object that represents the C
		     ; function.  If cname is a string "foo" these will be
		     ;
		     ;	  cname: "foo"  xname: foo*  sname: foo
		     ;
		     ; If cname is a pair ("foo" . bar) then we have
		     ;
		     ;    cname: "foo"   xname: bar*  sname: bar
		     
		     (or (string? name)
			 (and (pair? name) 
			      (string? (car name)) ; Name of C function
			      (symbol? (cdr name)) ; Name of Scheme function
			      )))
		(for-each parse-arg args)
		(letrec
		    ((defform 
		       (if *top-level-externs* 'define 'define-in-line))
		     (cname (if (string? name) name (car name)))
		     (xname (is (if (string? name) name (car name)) "*"))
		     (sname (is (if (string? name) name (cdr name))))
		     (emit-call
		      (lambda ()
			`(,xname 
			  ,@(map (lambda (x) (car (last-pair x))) args))))
		     (formals
		      (lambda (args)
			(if (null? args)
			    '()
			    (if (eq? (caar args) 'out)
				(formals (cdr args))
				(cons (car (last-pair (car args)))
				      (formals (cdr args))))))))
		  `(begin
		     (,(if builtin? 'define-c-builtin 'define-c-external)
		       (,xname ,@(map simple-type args))
		       ,(simple-type (list rtype 'returned))
		       ,cname)

		     (,defform (,sname ,@(formals args))
		       (let* (,@(map arg-in args)
			      (return-value
			       ,(cond
				 ((eq? rtype 'void)
				  `(begin ,(emit-call) #f))
				 ((eq? rtype 'bool)
				  `(not (= ,(emit-call) 0)))
				 ((eq? rtype 'string)
				  `(c-string->string ,(emit-call)))
				 ((isa-pointer? rtype)
				  `(cons ',(base-type rtype)
					 ,(emit-call)))
				 (else (emit-call)))))
			 ,(let ((out (args-out args)))
			    (if (null? out)
				'return-value
				(if (eq? rtype 'void)
				    (if (= (length out) 1)
					(car out)
					`(list ,@out))
				    `(list return-value ,@out)))))))
;		  (emit-extern-foo rtype cname args
;				   (if *top-level-externs*
;				       'define
;				       'define-in-line))
		  ))
	       (else
		(error 'extern "Illegal EXTERN syntax: (extern ~s ~s . ~s)"
		       rtype name args)))))
    ;;(format #t "(extern ~s ~s . ~s) ->~%" rtype name args)
    ;;(pp result (current-output-port)) (newline)
    result))

(define-macro (extern rtype name . args)
  `(do-extern #f ,rtype ,name ,@args))

(define-macro (builtin rtype name . args)
  `(do-extern #t ,rtype ,name ,@args))

;; This disables the extern and builtin macros when we're in the 
;; interpreter.  The normal definitions will just obscure the function
;; we're trying to gain access to. 

(eval-when (eval)
  (define-macro (extern . args) #f)
  (define-macro (builtin . args) #f))
