;;; Graphics Resource Functions

;;; 5.1.1  Creating, Copying, and Destroying Color Maps

(extern Colormap "XCreateColormap"
	(Display* dpy)
	(Window w)
	(Visual* visual)
	(int alloc))

(extern Colormap "XCopyColormapAndFree" (Display* dpy) (Colormap src_cmap))

(extern void "XSetWindowColormap"
	(Display* dpy)
	(Window w)
	(Colormap colormap))

(extern void "XFreeColormap" (Display* dpy) (Colormap cmap))

;;; 5.1.2  Allocating, Modifying, and Freeing Color Cells

(extern Status "XAllocColor"
	(Display* dpy)
	(Colormap cmap)
	(XColor* def))

(extern Status "XAllocNamedColor"
	(Display* dpy)
	(Colormap cmap)
	(string colorname)
	(out XColor hard_def) (out XColor exact_def))

(extern Status "XLookupColor"
	(Display* dpy)
	(Colormap cmap)
	(string spec)
	(out XColor def) (out XColor scr))

(extern Status "XAllocColorCells"
	(Display* dpy)
	(Colormap cmap)
	(Bool contig)
	(unsignedlongA* masks)
	(unsignedint nplanes)
	(unsignedlongA* pixels)
	(unsignedint ncolors))

(extern Status "XAllocColorPlanes"
	(Display* dpy)
	(Colormap cmap)
	(Bool contig)
	(unsignedA* pixels)
	(int ncolors)
	(int nreds) (int ngreens) (int nblues)
	(out unsignedlong rmask)
	(out unsignedlong gmask)
	(out unsignedlong bmask))

(extern void "XStoreColors"
	(Display* dpy)
	(Colormap cmap)
	(XColorA* defs)
	(int ncolors))

(extern void "XStoreColor" (Display* dpy) (Colormap cmap) (XColor* def))

(extern void "XStoreNamedColor"
	(Display* dpy)
	(Colormap cmap)
	(string name)
	(unsignedlong pixel)
	(int flags))

(extern void "XFreeColors"
	(Display* dpy)
	(Colormap cmap)
	(unsignedlongA* pixels)
	(int npixels)
	(unsignedlong planes))

;;; 5.1.3  Reading Entries in a Color Map

(extern void "XQueryColor"
	(Display* dpy)
	(Colormap cmap)
	(XColor* def))

(extern void "XQueryColors"
	(Display* dpy)
	(Colormap cmap)
	(XColorA* defs)
	(int ncolors))

;;; 5.2  Creating and Freeing Pixmaps

(extern Pixmap "XCreatePixmap"
	(Display* dpy)
	(Drawable d)
	(unsignedint width) (unsignedint height)
	(unsignedint depth))

(extern void "XFreePixmap" (Display* dpy) (Pixmap pixmap))

;;; 5.3 Manipulating Graphics Context/State

(extern GC "XCreateGC"
	(Display* dpy)
	(Drawable d)
	(unsignedlong valuemask)
	(XGCValues* values))

(extern void "XCopyGC"
	(Display* dpy)
	(GC srcGC)
	(unsignedlong mask)
	(GC destGC))

(extern void "XChangeGC"
	(Display* dpy)
	(GC gc)
	(unsignedlong valuemask)
	(XGCValues* values))

(extern void "XFreeGC" (Display* dpy) (GC gc))

(extern GContext "XGContextFromGC" (GC gc))

;;; 5.4.1  Setting the Foreground, Background, Function, or Plane Mask

(extern void "XSetState"
	(Display* dpy)
	(GC gc)
	(unsignedlong foreground)
	(unsignedlong background)
	(int function)
	(unsignedlong planemask))

(extern void "XSetForeground"
	(Display* dpy)
	(GC gc)
	(unsignedlong foreground))

(extern void "XSetBackground"
	(Display* dpy)
	(GC gc)
	(unsignedlong background))

(extern void "XSetFunction"
	(Display* dpy)
	(GC gc)
	(int function))

(extern void "XSetPlaneMask" (Display* dpy) (GC gc) (unsignedlong planemask))

;;; 5.4.2  Setting the Line Attributes and Dashes

(extern void "XSetLineAttributes"
	(Display* dpy)
	(GC gc)
	(unsignedint linewidth)
	(int linestyle)
	(int capstyle)
	(int joinstyle))

(extern void "XSetDashes"
	(Display* dpy)
	(GC gc)
	(int dash_offset)
	(charA* list)
	(int n))

;;; 5.4.3  Setting the Fill Style and Fill Rule

(extern void "XSetFillStyle" (Display* dpy) (GC gc) (int fill_style))

(extern void "XSetFillRule" (Display* dpy) (GC gc) (int fill_rule))

;;; 5.4.4  Setting the Fill Tile and Stipple

(extern Status "XQueryBestSize"
	(Display* dpy)
	(int class)
	(Drawable drawable)
	(unsignedint width) (unsignedint height)
	(out unsignedint ret_width) (out unsignedint ret_height))

(extern Status "XQueryBestTile"
	(Display* dpy)
	(Drawable drawable)
	(unsignedint width) (unsignedint height)
	(out unsignedint ret_width) (out unsignedint ret_height))

(extern Status "XQueryBestStipple"
	(Display* dpy)
	(Drawable drawable)
	(unsignedint width) (unsignedint height)
	(out unsignedint ret_width) (out unsignedint ret_height))

(extern void "XSetTile" (Display* dpy) (GC gc) (Pixmap tile))

(extern void "XSetStipple" (Display* dpy) (GC gc) (Pixmap stipple))

(extern void "XSetTSOrigin" (Display* dpy) (GC gc) (int x) (int y))

;;; 5.4.5  Setting the Current Font

(extern void "XSetFont" (Display* dpy) (GC gc) (Font font))

;;; 5.4.6  Setting the Clip Region

(extern void "XSetClipOrigin" (Display* dpy) (GC gc) (int xorig) (int yorig))

(extern void "XSetClipMask" (Display* dpy) (GC gc) (Pixmap mask))

(extern void "XSetClipRectangles"
	(Display* dpy)
	(GC gc)
	(int clip_x_origin) (int clip_y_origin)
	(XRectangleA* rectangles)
	(int n)
	(int ordering))

;;; 5.4.7  Setting the Arc Mode, Subwindow Mode, and Graphics Exposure

(extern void "XSetArcMode" (Display* dpy) (GC gc) (int arc_mode))

(extern void "XSetSubwindowMode" (Display* dpy) (GC gc) (int subwindow_mode))

(extern void "XSetGraphicsExposures"
	(Display* dpy)
	(GC gc)
	(Bool graphics_exposures))
