;;; Graphics Functions

;;; 6.1  Clearing Areas

(extern void "XClearWindow" (Display* dpy) (Window w))

(extern void "XClearArea"
	(Display* dpy)
	(Window w)
	(int x) (int y)
	(unsignedint width) (unsignedint height)
	(Bool exposures))

;;; 6.2  Copying Areas

(extern void "XCopyArea"
	(Display* dpy)
	(Drawable src_drawable)
	(Drawable dst_drawable)
	(GC gc)
	(int src_x) (int src_y)
	(unsignedint width) (unsignedint height)
	(int dst_x) (int dst_y))

(extern void "XCopyPlane"
	(Display* dpy)
	(Drawable src_drawable)
	(Drawable dst_drawable)
	(GC gc)
	(int src_x) (int src_y)
	(unsignedint width) (unsignedint height)
	(int dst_x) (int dst_y)
	(unsignedlong bit_plane))

;;; 6.3.1  Drawing Single and Multiple Points

(extern void "XDrawPoint"
	(Display* dpy)
	(Drawable d)
	(GC gc)
	(int x) (int y))

(extern void "XDrawPoints"
	(Display* dpy)
	(Drawable d)
	(GC gc)
	(XPointA* points)
	(int n_points)
	(int mode))

;;; 6.3.2  Drawing Single and Multiple Lines

(extern void "XDrawLine"
	(Display* dpy)
	(Drawable d)
	(GC gc)
	(int x1) (int y1)
	(int x2) (int y2))

(extern void "XDrawLines"
	(Display* dpy)
	(Drawable d)
	(GC gc)
	(XPointA* points)
	(int npoints)
	(int mode))

(extern void "XDrawSegments"
	(Display* dpy)
	(Drawable d)
	(GC gc)
	(XSegmentA* segments)
	(int nsegments))

;;; 6.3.3  Drawing Single and Multiple Rectangles

(extern void "XDrawRectangle"
	(Display* dpy)
	(Drawable d)
	(GC gc)
	(int x) (int y)
	(unsignedint width) (unsignedint height))

(extern void "XDrawRectangles"
	(Display* dpy)
	(Drawable d)
	(GC gc)
	(XRectangleA* rects)
	(int n_rects))

;;; 6.3.4  Drawing Single and Multiple Arcs

(extern void "XDrawArc"
	(Display* dpy)
	(Drawable d)
	(GC gc)
	(int x) (int y)
	(unsignedint width) (unsignedint height)
	(int angle1) (int angle2))

(extern void "XDrawArcs"
	(Display* dpy)
	(Drawable d)
	(GC gc)
	(XArcA* arcs)
	(int n_arcs))

;;; 6.4.1  Filling Single and Multiple Rectangles

(extern void "XFillRectangle"
	(Display* dpy)
	(Drawable d)
	(GC gc)
	(int x) (int y)
	(unsignedint width) (unsignedint height))

(extern void "XFillRectangles"
	(Display* dpy)
	(Drawable d)
	(GC gc)
	(XRectangleA* rectangles)
	(int n_rects))

;;; 6.4.2  Filling a Single Polygon

(extern void "XFillPolygon"
	(Display* dpy)
	(Drawable d)
	(GC gc)
	(XPointA* points)
	(int n_points)
	(int shape)
	(int mode))

;;; 6.4.3  Filling Single and Multiple Arcs

(extern void "XFillArc"
	(Display* dpy)
	(Drawable d)
	(GC gc)
	(int x) (int y)
	(unsignedint width) (unsignedint height)
	(int angle1) (int angle2))

(extern void "XFillArcs"
	(Display* dpy)
	(Drawable d)
	(GC gc)
	(XArcA* arcs)
	(int n_arcs))

;;; 6.5.1  Loading and Freeing Fonts

(extern Font "XLoadFont" (Display* dpy) (string name))

(extern XFontStruct* "XQueryFont" (Display* dpy) (Font fid))

(extern XFontStruct* "XLoadQueryFont" (Display* dpy) (string name))

(extern void "XFreeFont" (Display* dpy) (XFontStruct* fs))

(extern Bool "XGetFontProperty"
	(XFontStruct* fs)
	(Atom name)
	(out unsignedlong valuePtr))

(extern void "XUnloadFont" (Display* dpy) (Font font))

;;; 6.5.2  Obtaining and Freeing Font Names and Information

(extern char*A* "XListFonts"
	(Display* dpy)
	(string pattern)
	(int maxnames)
	(out int actual_count))

(extern void "XFreeFontNames" (char*A* list))

(extern char*A* "XListFontsWithInfo"
	(Display* dpy)
	(string pattern)
	(int maxnames)
	(out int actual_count)
	(out XFontStructA* info))

(extern void "XFreeFontInfo" (char*A* names) (XFontStructA* info) (int count))

;;; 6.5.3  Setting and Retrieving the Font Search Path

(extern void "XSetFontPath"
	(Display* dpy)
	(char*A* directories)
	(int ndirs))

(extern char*A* "XGetFontPath" (Display* dpy) (int npaths))

(extern char*A* "XFreeFontPath" (char*A* list))

;;; 6.5.4  Computing Character String Sizes

(extern int "XTextWidth"
	(XFontStruct* fontstruct)
	(string string)
	(int count))

(extern int "XTextWidth16"
	(XFontStruct* fontstruct)
	(XChar2bA* string)
	(int count))

;;; 6.5.5  Computing Logical Extents

(extern void "XTextExtents"
	(XFontStruct* fontstruct)
	(string string)
	(int nchars)
	(out int dir)
	(out int font_ascent) (out int font_descent) (out XCharStruct overall))

(extern void "XTextExtents16"
	(XFontStruct* fontstruct)
	(XChar2bA* string)
	(int nchars)
	(out int dir)
	(out int font_ascent) (out int font_descent) (out XCharStruct overall))

;;; 6.5.6  Querying Character String Sizes

(extern void "XQueryTextExtents"
	(Display* dpy)
	(Font fid)
	(string string)
	(int nchars)
	(out int dir)
	(out int font_ascent) (out int font_descent) (out XCharStruct overall))

(extern void "XQueryTextExtents16"
	(Display* dpy)
	(Font fid)
	(XChar2bA* string)
	(int nchars)
	(out int dir)
	(out int font_ascent) (out int font_descent) (out XCharStruct overall))

;;; 6.6.1  Drawing Complex Text

(extern void "XDrawText"
	(Display* dpy)
	(Drawable d)
	(GC gc)
	(int x)
	(int y)
	(XTextItemA* items)
	(int nitems))

(extern void "XDrawText16"
	(Display* dpy)
	(Drawable d)
	(GC gc)
	(int x)
	(int y)
	(XTextItem16* items)
	(int nitems))

;;; 6.6.2  Drawing Text Characters

(extern void "XDrawString"
	(Display* dpy)
	(Drawable d)
	(GC gc)
	(int x)
	(int y)
	(string string)
	(int length))

(extern void "XDrawString16"
	(Display* dpy)
	(Drawable d)
	(GC gc)
	(int x)
	(int y)
	(XChar2bA* string)
	(int length))

;;; 6.6.3  Drawing Image Text Characters

(extern void "XDrawImageString"
	(Display* dpy)
	(Drawable d)
	(GC gc)
	(int x) (int y)
	(string string)
	(int length))

(extern void "XDrawImageString16"
	(Display* dpy)
	(Drawable d)
	(GC gc)
	(int x) (int y)
	(XChar2bA* string)
	(int length))

;;; 6.7  Transferring Images between Client and Server

(extern void "XPutImage"
	(Display* dpy)
	(Drawable d)
	(GC gc)
	(XImage* image)
	(int req_xoffset)
	(int req_yoffset)
	(int x)
	(int y)
	(unsignedint req_width)
	(unsignedint req_height))

(extern XImage* "XGetImage"
	(Display* dpy)
	(Drawable d)
	(int x)
	(int y)
	(unsignedint width)
	(unsignedint height)
	(unsignedlong plane_mask)
	(int format))

(extern XImage* "XGetSubImage"
	(Display* dpy)
	(Drawable d)
	(int x)
	(int y)
	(unsignedint width)
	(unsignedint height)
	(unsignedlong plane_mask)
	(int format)
	(XImage* dest_image)
	(int dest_x)
	(int dest_y))

;;; 6.8.1 Creating a Cursor

(extern Cursor "XCreateFontCursor"
	(Display* dpy)
	(unsignedint which))

(extern Cursor "XCreatePixmapCursor"
	(Display* dpy)
	(Pixmap source)
	(Pixmap mask)
	(XColor* foreground)
	(XColor* background)
	(unsignedint x)
	(unsignedint y))

(extern Cursor "XCreateGlyphCursor"
	(Display* dpy)
	(Font source_font)
	(Font mask_font)
	(unsignedint source_char)
	(unsignedint mask_char)
	(XColor* foreground)
	(XColor* background))

;;; 6.8.2  Changing and Destroying Cursors

(extern void "XRecolorCursor"
	(Display* dpy)
	(Cursor cursor)
	(XColor* foreground)
	(XColor* background))

(extern void "XFreeCursor"
	(Display* dpy)
	(Cursor cursor))

(extern Status "XQueryBestCursor"
	(Display* dpy)
	(Drawable drawable)
	(unsignedint width)
	(unsignedint height)
	(out unsignedint ret_width)
	(out unsignedint ret_height))

;;; 6.8.3  Defining the Cursor

(extern void "XDefineCursor"
	(Display* dpy)
	(Window w)
	(Cursor cursor))

(extern void "XUndefineCursor"
	(Display* dpy)
	(Window w))
